#!/usr/bin/env python3

import os
import sys

if "__file__" in locals():
    sys.path.insert(0, os.path.realpath(os.path.join( os.path.dirname(os.path.realpath(__file__)),'..','..')))

import subprocess
import locale

import zipfile
import datetime

from optparse import OptionParser
import waptserver.config
from waptserver.config import type_debian, type_redhat, get_distrib_major


if type_debian():
    PGSQL_SVC = 'postgresql'
    wapt_folder = '/var/www/wapt'
#    NGINX_GID = grp.getgrnam('www-data').gr_gid
elif type_redhat():
    PGSQL_SVC = 'postgresql' if get_distrib_major() == '8' else 'postgresql-9.6'
    wapt_folder = '/var/www/html/wapt'
#    NGINX_GID = grp.getgrnam('nginx').gr_gid


def run(*args, **kwargs):
    return subprocess.check_output(*args, shell=True, **kwargs).decode(locale.getpreferredencoding())


def run_verbose(*args, **kwargs):
    output = run(*args, **kwargs)
    print(output)
    return output


def mkdir(path):
    if not os.path.isdir(path):
        os.makedirs(path)

def start_waptserver():
    out = run("systemctl restart waptserver")
    out += run("systemctl restart wapttasks")
    return out

def stop_waptserver():
    out = run("systemctl stop waptserver")
    out += run("systemctl stop waptserver")

def start_postgresql():
    out = run("systemctl stop postgresql")
    out += run("systemctl stop postgresql")

def stop_postgresql():
    out = run("systemctl stop postgresql")
    out += run("systemctl stop postgresql")



def main():

    usage = """ %prog [--config filename] [--full-backup] [--config-backup] [--package-backup] [--dest-backup] """


    parser = OptionParser(usage=usage)

    parser.add_option(
        '-c',
        '--config',
        dest='configfile',
        default=waptserver.config.DEFAULT_CONFIG_FILE,
        help='Config file full path (default: %default)')
    parser.add_option(
        '--full-backup',
        dest='full_backup',
        default=None,
        action='store_true',
        help='Run full backup of WaptServer including configfiles, keys, database and packages.')
    parser.add_option(
        '--config-backup',
        dest='config_backup',
        default=None,
        action='store_true',
        help='Run backup of WaptServer including only configfiles, keys and database. Packages are exclude.')
    parser.add_option(
        '--package-backup',
        dest='package_backup',
        default=None,
        action='store_true',
        help='Run backup of Wapt Packages only.')
    parser.add_option(
        '-d',
        '--dest-backup',
        dest='dest_backup',
        default='/tmp',
        action='store_true',
        help='Full path where store backup')

    (options, args) = parser.parse_args()


    server_config = waptserver.config.load_config(options.configfile)

    def config_backup(zipBackup):
        if server_config['use_kerberos']:
            print('Backup file /etc/nginx/http-krb5.keytab')
            zipBackup.write('/etc/nginx/http-krk5.keytab')

        for root, dirs, files in os.walk('/etc/nginx/sites-enabled'):
            for f in files:
                print('Backup file %s' % f)
                zipBackup.write(os.path.join(root,f))

        for dirs in ['conf','waptserver/ssl']:
            for root, dirs, files in os.walk('/opt/wapt/%s' % dirs):
                for f in files:
                    print('Backup file %s' %f)
                    zipBackup.write(os.path.join(root,f))

        print('Start backup waptdb')
        run("sudo -u postgres pg_dumpall > /tmp/backup_waptdb.sql")
        zipBackup.write('/tmp/backup_waptdb.sql')
        os.remove('/tmp/backup_waptdb.sql')
        print('Finish backup waptdb')


    def package_backup(zipBackup):
        for root, dirs, files in os.walk(server_config['wapt_folder']):
            for f in files:
                print('Backup file %s' % f)
                zipBackup.write(os.path.join(root,f))

    print('Start WAPT Backup')
    print('Backup dir : ', options.dest_backup)

    datetime_now = datetime.datetime.utcnow()
    backup_name = os.path.join(options.dest_backup,'%s_wapt.zip' % datetime_now.isoformat())
    zipBackup = zipfile.ZipFile(backup_name, 'w')

    if options.full_backup:
        config_backup(zipBackup)
        package_backup(zipBackup)

    elif options.config_backup:
        config_backup(zipBackup)

    elif options.package_backup:
        package_backup(zipBackup)

    else:
        print('No backup type. Exitting...')
        sys.exit(2)

    zipBackup.close()

    print('\n\nWaptServer successfuly backup in %s' % (backup_name))


if __name__ == '__main__':
    main()