import os
import sys
import dialog
from optparse import OptionParser

if "__file__" in locals():
    sys.path.insert(0, os.path.realpath(os.path.join( os.path.dirname(os.path.realpath(__file__)),'..','..')))

from setuphelpers import *
import waptserver.config

postconf = dialog.Dialog(dialog="dialog")

def compile_ipxe(tmpdir='/tmp',config='/opt/wapt/conf/waptserver.ini',ip=''):

    if not ip:
        # not work with dns ...
        msg = 'IP OF WAPT server (eg. 192.168.1.10)'
        (code, reply) = postconf.inputbox(text=msg, width=len(msg)+4, init=networking()[0]['addr'][0]['addr'])
        if code != postconf.DIALOG_OK:
            exit(1)
        else:
            ip = reply

    if type_debian():
        list_package = ['git','build-essential','zlib1g-dev','binutils-dev','gcc','liblzma-dev','make']
        print('install %s' % ' '.join(list_package))
        install_apt(' '.join(list_package))
    elif type_redhat():
        list_package = ['git', 'gcc', 'gcc-c++', 'make', 'zlib-devel', 'binutils-devel' ,'xz-devel']
        print('install %s' % ' '.join(list_package))
        install_yum(' '.join(list_package))

    else :
        print('not supported os')
        sys.exit(1)

    mkdirs('%s/ipxe' % tmpdir)
    run('git clone https://github.com/ipxe/ipxe.git %s/ipxe' % tmpdir)

    print('custom ipxe')
    with open('%s/ipxe/src/config/console.h' % tmpdir,'r') as f:
        data = f.read()
    data = data.replace('#define	KEYBOARD_MAP	us','#define	KEYBOARD_MAP	dynamic')
    with open('%s/ipxe/src/config/console.h' % tmpdir,'w') as f:
        f.write(data)

    with open('%s/ipxe/src/legacy.ipxe' % tmpdir,'w') as f:
        f.write("#!ipxe\n\ndhcp\nchain http://%s/api/v3/baseipxe?uefi=false" % ip)

    with open('%s/ipxe/src/uefi.ipxe' % tmpdir,'w') as f:
        f.write("#!ipxe\n\ndhcp\nchain http://%s/api/v3/baseipxe" % ip)

    print('compile legacy ipxe')
    run('cd %s/ipxe/src ; make bin/undionly.kpxe EMBED=legacy.ipxe' % tmpdir)

    print('compile uefi ipxe')
    run('cd %s/ipxe/src ; make bin-x86_64-efi/ipxe.efi EMBED=uefi.ipxe' % tmpdir)

    print('compile snponly.efi')
    run('cd %s/ipxe/src ; make bin-x86_64-efi/snponly.efi EMBED=uefi.ipxe' % tmpdir)

    server_config = waptserver.config.load_config(config)
    wads_folder = server_config['wads_folder']
    pxe_folder= makepath(wads_folder,'pxe')

    filecopyto('%s/ipxe/src/bin/undionly.kpxe' % tmpdir,'%s/undionly_custom.kpxe' % pxe_folder)
    filecopyto('%s/ipxe/src/bin-x86_64-efi/ipxe.efi' % tmpdir, '%s/ipxe_custom.efi' % pxe_folder)
    filecopyto('%s/ipxe/src/bin-x86_64-efi/snponly.efi' % tmpdir, '%s/snponly_custom.efi' % pxe_folder)
    remove_tree('%s/ipxe' % tmpdir)
    systemd_restart_service('wapttftpserver')

def main():

    usage = """%prog [--config filename] [--ip ip server]"""

    parser = OptionParser(usage=usage, version=__version__)
    parser.add_option(
        '-c',
        '--config',
        dest='configfile',
        default='/opt/wapt/conf/waptserver.ini',
        help='Config file full path (default: %default)')

    parser.add_option(
        '-i',
        '--ip',
        dest='ip',
        default='',
        help='ip for waptserver server (default: ask in gui)')


    (options, args) = parser.parse_args()
    compile_ipxe(config=options.configfile,ip=options.ip)

if __name__ == "__main__":
    if not type_debian() and not type_redhat():
        print("Unsupported distribution")
        sys.exit(1)
    main()
