#!/usr/bin/env python3
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2023  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##

import os
import sys

if "__file__" in locals():
    sys.path.insert(0, os.path.realpath(os.path.join( os.path.dirname(os.path.realpath(__file__)),'..','..')))

__version__ = "1.4.0"

import subprocess
import socket
import getpass
import sys
import os
from optparse import OptionParser

try:
    import distro
    linux_distribution = distro.linux_distribution(False)
except ImportError:
    linux_distribution = (None, None, None)

usage = """config-krb5.py <option>"""


parser = OptionParser(usage=usage, version="1.4.0")
parser.add_option("-d", "--dc-name", dest="dc_name", default=None, help="DC to join to (default: %default)")

(options, args) = parser.parse_args()


def type_redhat():
    return linux_distribution is not None and linux_distribution[0].lower() in ('rhel', 'centos', 'fedora')


def type_debian():
    return linux_distribution is not None and linux_distribution[0].lower() in ('debian', 'ubuntu', 'linuxmint', 'raspbian')


def main(dc_name):
    fqdn = socket.getfqdn()

    # check if specified dc is pingable
    if dc_name:
        response = os.system("ping -c 1 %s " % dc_name)
        # and then check the response...
        if response != 0:
            print("couldn't not ping dc %s , please check network configuration" % dc_name)
            sys.exit(1)

    # TODO : should try to ping dc_name

    cmd = """msktutil --precreate --host %s -b cn=computers --service HTTP --description "Host account for WaptServer" --enctypes 24  """ % (fqdn,)
    if dc_name != '':
        cmd = "%s --server %s" % (cmd, dc_name)

    print("Executing shell command : %s " % cmd)
    print(subprocess.check_output(cmd, shell=True))

    # --server needed only if default dc not suitable for RW operation (rodc, etc.)
    # we support only 0x8=aes128-cts-hmac-sha1 and  0x10=aes256-cts-hmac-sha1 (no DES / RC4)

    # create keytab
    cmd = """msktutil --auto-update --keytab /etc/apache2/http-krb5.keytab --host %s""" % (fqdn,)

    if dc_name != '':
        cmd = "%s --server %s" % (cmd, dc_name)

    print("Executing shell command : %s " % cmd)
    print(subprocess.check_output(cmd, shell=True))

    print("Fixing right and ownership")
    print(subprocess.check_output("chown www-data:root /etc/apache2/http-krb5.keytab", shell=True))
    print(subprocess.check_output("chmod 600 /etc/apache2/http-krb5.keytab", shell=True))


if __name__ == "__main__":
    if not type_debian() and not type_redhat():
        print("unsupported distrib")
        sys.exit(1)

    if getpass.getuser() != 'root':
        print("Command should be run as root")
        sys.exit(1)

    main(options.dc_name)
