#!/usr/bin/env python3
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2023  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##
from flask import request, Blueprint

from waptserver.decorators import requires_auth, requires_enterprise

from waptserver.server_socketio import proxy_host_request

wsus_socketio = Blueprint('wsus_socketio', __name__)

try:
    # i18n
    from flask_babel import Babel
    from flask_babel import gettext
except ImportError:
    gettext = (lambda s: s)
    Babel = None

_ = gettext


@wsus_socketio.route('/api/v3/trigger_waptwua_scan', methods=['HEAD', 'GET', 'POST'])
@requires_enterprise
@requires_auth(['admin','view'])
def trigger_waptwua_scan():
    """Proxy the wapt update action to the client"""
    return proxy_host_request(request, 'trigger_waptwua_scan')


@wsus_socketio.route('/api/v3/trigger_waptwua_download', methods=['HEAD', 'GET', 'POST'])
@requires_enterprise
@requires_auth(['admin','view'])
def trigger_waptwua_download():
    """Proxy the wapt download action to the client"""
    return proxy_host_request(request, 'trigger_waptwua_download')


@wsus_socketio.route('/api/v3/trigger_waptwua_install', methods=['HEAD', 'GET', 'POST'])
@requires_enterprise
@requires_auth(['admin','view'])
def trigger_waptwua_install():
    """Proxy the wapt scan action to the client"""
    return proxy_host_request(request, 'trigger_waptwua_install')
